
// WhitenDoc.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, Oct. 2022

/*

   run WhitenDoc elementsofcoordi00lone.pdf 80 152
   452 pages; 2.2 GB used max
   Saving whitened doc in elementsofcoordi00lone-WAll.pdf
   Time taken: 1180 secs
   21 MB --> 46 MB

   run WhitenDoc "11th Vol 6.pdf" 23 203
   1028 pages; 2.2 GB used max
   Saving whitened doc in 11th Vol 6-WAll.pdf
   Time taken: 568 secs
   132 MB --> 397 MB
*/ 

import java.io.*;
import java.awt.image.BufferedImage;

import org.apache.pdfbox.pdmodel.*;
import org.apache.pdfbox.pdmodel.common.*;
import org.apache.pdfbox.pdmodel.graphics.image.*;
import org.apache.pdfbox.rendering.*;

import ij.*;
import ij.process.*;
import ij.plugin.ContrastEnhancer;



public final class WhitenDoc
{
  private static final int DPI_DEFAULT = 120;
  private static final double CONTRAST_DEFAULT = 0.8;


  public static void main(String args[])
  {
    boolean useContrast = false;
    double min = 0;
    double max = 0;

    if (args.length == 1)
      useContrast = true;
    else if (args.length == 3) {
      min = Double.parseDouble(args[1]);
      max = Double.parseDouble(args[2]);
    }
    else {
      System.out.println("Usage: run WhitenPage <fnm> [<min> <Max>]");
      System.exit(0);
    }

    String fnm = args[0];

    long start = System.currentTimeMillis();

    try {
      PDDocument doc = PDDocument.load(new File(fnm));
      int lastPage = doc.getNumberOfPages();
      System.out.println("No. of pages: " + lastPage);

      PDDocument wDoc = new PDDocument();

      PDFRenderer renderer = new PDFRenderer(doc);
      renderer.setSubsamplingAllowed(true);

      for (int i=0; i < lastPage; i++) {
        System.out.print(" " + (i+1));
        BufferedImage im = renderer.renderImageWithDPI(i, DPI_DEFAULT, ImageType.RGB); 
        // System.out.println("- Image (w,h): " + im.getWidth() + ", " + im.getHeight());
	    
        BufferedImage whitenedIm = whitenImage(im, useContrast, min, max);    
	    
        addImagePage(wDoc, whitenedIm);
        if ((i+1)%15 == 0)
          System.out.println();
      }
      System.out.println();
      doc.close();

      String outFnm = stripExt(fnm) + "-WAll.pdf";
      System.out.println("Saving whitened doc in " + outFnm);
      wDoc.save(outFnm);
      wDoc.close();

      long end = System.currentTimeMillis(); 
      System.out.println("Time taken: " + ((end-start)/1000) +
              " secs" );
    }
    catch(Exception e)
    {  System.out.println(e);  }

  }  // end of main()



  private static BufferedImage whitenImage(BufferedImage im, 
                   boolean useContrast, double min, double max) 
                               throws Exception
  {
    // convert to grayscale and enhance contrast
    ImagePlus imp = new ImagePlus("Page Image", im);
    ImageConverter ic = new ImageConverter(imp);
    ic.convertToGray8();
    // System.out.println("- converted to 8-bit grayscale");
    
    if (useContrast) {
      ContrastEnhancer ce = new ContrastEnhancer();
      ce.stretchHistogram(imp, CONTRAST_DEFAULT);
      // System.out.println("- whiten using contrast: " + CONTRAST_DEFAULT);
    }
    else {
      ImageProcessor ip = imp.getProcessor();
      ip.setMinAndMax(min, max);
      // System.out.println("- whiten using min-max: " + min + " - " + max);
    }

    return imp.getBufferedImage();       
  }  // end of whitenImage()


  private static void addImagePage(PDDocument doc, BufferedImage im)
                                                throws Exception
  {
    // System.out.println("- converting image to PDF page");
    PDPage page = new PDPage(new PDRectangle(im.getWidth(), im.getHeight()));
    doc.addPage(page); 

    PDImageXObject xImg = JPEGFactory.createFromImage(doc, im);

    PDPageContentStream contents = new PDPageContentStream(doc, page);
    contents.drawImage(xImg, 0, 0);
    contents.close();
  }  // end of addImagePage()



  private static String stripExt(String fnm)
  {
    int dotPos = fnm.lastIndexOf('.');
    if (dotPos == -1)
      return fnm;
    else
      return fnm.substring(0, dotPos);
  }


} // end  of WhitenDoc class

