
// WhitenPage.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, Oct. 2022

/*
  > run WhitenPage elements20.pdf 5
  Executing WhitenPage elements20.pdf 5 with PDFBox and ImageJ...
  Extracting page 5...
  Image (w,h): 991, 1403
  - converted to 8-bit grayscale
  - whiten using contrast: 0.8
  Saving image page in elements20-W5.pdf
  
  > run WhitenPage elements20.pdf 5 23 178
  Executing WhitenPage elements20.pdf 5 23 178 with PDFBox and ImageJ...
  Extracting page 5...
  Image (w,h): 991, 1403
  - converted to 8-bit grayscale
  - whiten using min-max: 23.0 - 178.0
  Saving image page in elements20-W5.pdf

*/

import java.io.*;
import java.awt.image.BufferedImage;

import org.apache.pdfbox.pdmodel.*;
import org.apache.pdfbox.pdmodel.common.*;
import org.apache.pdfbox.pdmodel.graphics.image.*;
import org.apache.pdfbox.rendering.*;

import ij.*;
import ij.process.*;
import ij.plugin.ContrastEnhancer;


public final class WhitenPage
{
  private static final int DPI_DEFAULT = 120;
  private static final double CONTRAST_DEFAULT = 0.8;


  public static void main(String args[])
  {
    boolean useContrast = false;
    double min = 0;
    double max = 0;

    if (args.length == 2)
      useContrast = true;
    else if (args.length == 4) {
      min = Double.parseDouble(args[2]);
      max = Double.parseDouble(args[3]);
    }
    else {
      System.out.println("Usage: run WhitenPage <fnm> <pageNo> [<min> <Max>]");
      System.exit(0);
    }

    if ((min < 0) || (min > max)) {
      System.out.println("Incorrect min - max: " + min + " - " + max);
      System.exit(0);
    }

    String fnm = args[0];
    int pageIdx = Integer.parseInt(args[1]) - 1;

    try {
      PDDocument doc = PDDocument.load(new File(fnm));
      int lastPage = doc.getNumberOfPages();

      if ((pageIdx < 0) || (pageIdx >= lastPage)) {
        System.out.println("Page No. out of range: 1 - " + lastPage);
        System.exit(0);
      }

      System.out.println("Extracting page " + (pageIdx+1) + "...");
      PDFRenderer renderer = new PDFRenderer(doc);
      renderer.setSubsamplingAllowed(true);

      BufferedImage im = renderer.renderImageWithDPI(pageIdx, DPI_DEFAULT, ImageType.RGB); 
      System.out.println("Image (w,h): " + im.getWidth() + ", " + im.getHeight());

      BufferedImage whitenedIm = whitenImage(im, useContrast, min, max);    
   
      doc.close();

      // save image on a page in a pdf file
      PDDocument wDoc = new PDDocument();
      addImagePage(wDoc, whitenedIm);
      String outFnm = stripExt(fnm) + "-W" + (pageIdx+1) + ".pdf";
      System.out.println("Saving image page in " + outFnm);
      wDoc.save(outFnm);
      wDoc.close();
    }
    catch(Exception e)
    {  System.out.println(e);  }

  }  // end of main()


  private static BufferedImage whitenImage(BufferedImage im, 
                   boolean useContrast, double min, double max) 
                               throws Exception
  {
    // convert to grayscale and enhance contrast
    ImagePlus imp = new ImagePlus("Page Image", im);
    ImageConverter ic = new ImageConverter(imp);
    ic.convertToGray8();
    System.out.println("- converted to 8-bit grayscale");
    
    if (useContrast) {
      ContrastEnhancer ce = new ContrastEnhancer();
      ce.stretchHistogram(imp, CONTRAST_DEFAULT);
      System.out.println("- whiten using contrast: " + CONTRAST_DEFAULT);
    }
    else {
      ImageProcessor ip = imp.getProcessor();
      ip.setMinAndMax(min, max);
      System.out.println("- whiten using min-max: " + min + " - " + max);
    }

    return imp.getBufferedImage();       
  }  // end of whitenImage()



  private static void addImagePage(PDDocument doc, BufferedImage im)
                                                throws Exception
  {
    PDPage page = new PDPage(new PDRectangle(im.getWidth(), im.getHeight()));
    doc.addPage(page); 

    PDImageXObject xImg = JPEGFactory.createFromImage(doc, im);

    PDPageContentStream contents = new PDPageContentStream(doc, page);
    contents.drawImage(xImg, 0, 0);
    contents.close();
  }  // end of addImagePage()



  private static String stripExt(String fnm)
  {
    int dotPos = fnm.lastIndexOf('.');
    if (dotPos == -1)
      return fnm;
    else
      return fnm.substring(0, dotPos);
  }


} // end  of WhitenPage class

